/*
 * Copyright (C) 2007 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.example.android.snake;

import android.app.Activity;
import android.os.Bundle;
import android.widget.FrameLayout;
import android.widget.FrameLayout.LayoutParams;
import android.widget.TextView;

/** Test add gestures handle **/
import android.view.Gravity;
import android.view.KeyEvent;
import android.view.View;
import android.view.MotionEvent;
import android.gesture.GestureOverlayView;
import android.gesture.Gesture;

/**
 * Snake: a simple game that everyone can enjoy.
 * 
 * This is an implementation of the classic Game "Snake", in which you control a
 * serpent roaming around the garden looking for apples. Be careful, though,
 * because when you catch one, not only will you become longer, but you'll move
 * faster. Running into yourself or the walls will end the game.
 * 
 */
public class Snake extends Activity implements GestureOverlayView.OnGestureListener  {

    private static final float LENGTH_THRESHOLD = 120.0f;
    private SnakeView mSnakeView;
    private Gesture mGesture;
    private TextView tv = null; 
    private long startX = 0; 
    private long startY = 0; 
    private long endX = 0; 
    private long endY = 0; 

    
    private static String ICICLE_KEY = "snake-view";
    

    public void onGestureCancelled(GestureOverlayView overlay, MotionEvent event) {
//    	print(overlay, event, "onGestureCancelled");
    	endX = (long) event.getX();
    	endY = (long) event.getY();
    	simulateKey (startX, startY, endX, endY);    	
    }

    public void onGestureEnded(GestureOverlayView overlay, MotionEvent event) {
//    	print(overlay, event, "onGestureEnded");
    	endX = (long) event.getX();
    	endY = (long) event.getY();
    	simulateKey (startX, startY, endX, endY);    	
    	
    }

    public void onGestureStarted(GestureOverlayView overlay, MotionEvent event) {
//    	print(overlay, event, "onGestureStarted");
    	startX = (long) event.getX();
    	startY = (long) event.getY();
    }

	public void onGesture(GestureOverlayView overlay, MotionEvent event) {
//		print(overlay, event, "onGesture");
        if (mGesture != null  && mGesture.getLength() < LENGTH_THRESHOLD) {
            overlay.clear(false);
        }
	}
        

    /**
     * Called when Activity is first created. Turns off the title bar, sets up
     * the content views, and fires up the SnakeView.
     * 
     */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        setContentView(R.layout.snake_layout);

        mSnakeView = (SnakeView) findViewById(R.id.snake);
        mSnakeView.setTextView((TextView) findViewById(R.id.text));

	   GestureOverlayView overlay = (GestureOverlayView) findViewById(R.id.gestures_overlay);
	   overlay.addOnGestureListener(this);
                
        if (savedInstanceState == null) {
            // We were just launched -- set up a new game
            mSnakeView.setMode(SnakeView.READY);
        } else {
            // We are being restored
            Bundle map = savedInstanceState.getBundle(ICICLE_KEY);
            if (map != null) {
                mSnakeView.restoreState(map);
            } else {
                mSnakeView.setMode(SnakeView.PAUSE);
            }
        }
    }

    @Override
    protected void onPause() {
        super.onPause();
        // Pause the game along with the activity
        mSnakeView.setMode(SnakeView.PAUSE);
    }

    @Override
    public void onSaveInstanceState(Bundle outState) {
        //Store the game state
        outState.putBundle(ICICLE_KEY, mSnakeView.saveState());
    }


    
    private void simulateKey (long startX, long startY, long endX, long endY) {
    	
    	
    	long pathX = startX - endX; 
    	long pathY = startY - endY;
    	
    	if (Math.abs(pathX) > Math.abs(pathY))  {
    		// EAST or WEST
        	if (startX > endX)  {
        		// WEST
        		mSnakeView.onKeyDown( KeyEvent.KEYCODE_DPAD_LEFT, new KeyEvent(KeyEvent.ACTION_DOWN, KeyEvent.KEYCODE_DPAD_LEFT ));
        		
        	} else {
        		// EAST 
        		mSnakeView.onKeyDown( KeyEvent.KEYCODE_DPAD_RIGHT, new KeyEvent(KeyEvent.ACTION_DOWN, KeyEvent.KEYCODE_DPAD_RIGHT ));

        	}
    		
    	} 
    		
    	if (Math.abs(pathX) < Math.abs(pathY))  {
    		// NORTH or SOUTH
        	if (startY < endY)  {
        		// SOUTH
        		mSnakeView.onKeyDown( KeyEvent.KEYCODE_DPAD_DOWN, new KeyEvent(KeyEvent.ACTION_DOWN, KeyEvent.KEYCODE_DPAD_DOWN ));
        		
        	} else {
        		// NORTH 
        		mSnakeView.onKeyDown( KeyEvent.KEYCODE_DPAD_UP, new KeyEvent(KeyEvent.ACTION_DOWN, KeyEvent.KEYCODE_DPAD_UP ));
        	}
    		
    	} 
    	
    }
    
    
    private void print( GestureOverlayView gd, MotionEvent evt, String msg) {
    	FrameLayout g = (FrameLayout)findViewById(R.id.snake_layout);
        if (this.tv != null) {
            g.removeView(tv);
        	}
        
    	if (this.tv == null) { 
    		tv = new TextView(this);
    		}
    	
        if (this.tv != null) {
        		
    		g. addView(tv);
    		LayoutParams lp = (LayoutParams) tv.getLayoutParams();
            lp.height = 50; //LayoutParams.MATCH_PARENT;
            lp.width = 200;
            tv.setLayoutParams(lp);
            tv.setGravity(Gravity.CENTER_HORIZONTAL | Gravity.CENTER_VERTICAL);
            tv.setBackgroundColor(0xff888888);
            tv.setText("HERE : " + evt.getX() + "/" +  evt.getY() + "\n" + msg);
            tv.setVisibility(View.VISIBLE);
    		}
    	}
    }

   
    
